<?php
namespace plugins\file_manager\controllers;

use plugins\file_manager\library\Breadcrumbs;
use plugins\file_manager\library;
use plugins\file_manager\services\FileSystem;
use plugins\file_manager\library\exceptions;

include( dirname( dirname( __FILE__ ) ) . DS . 'library' . DS . 'base64url.php' );

/**
 * File controller
 * @author Jérôme Brilland
 * @version 2.1
 * @copyright (c) 2021-2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class FileController extends \ActionController {

    use Breadcrumbs;

    private $filesystem;
    private $fullpath;

    protected function beforeFilter() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'file_manager.manage_files' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'files' );
        $this->filesystem = new FileSystem( $this );
        $this->decodedPath = ( !empty( $this->request->getAttribute( 'path', '' ) ) ) ? library\base64url_decode( $this->request->getAttribute( 'path' ) ) : '';
        $this->fullpath = PUBLIC_DIR . $this->decodedPath;
        $this->encodedPath = $this->request->getAttribute( 'path', '' );
    }

    public function doCreate() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $filename = $this->request->getParameters()->getString( 'filename', null, 'unsafe_raw', FILTER_FLAG_STRIP_LOW );
            $this->filesystem->createFile( $this->fullpath . DS . $filename );
        } catch ( exceptions\FileExistsException $e ) {
            $this->flash['error'] = $i18n->__( 'file_already_exists', $filename );
        } catch ( Exception $e ) {
            $this->flash['error'] = $i18n->__( 'file_not_created', $filename );
        } finally {
            $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( $this->decodedPath ) );
        }
    }

    public function doDelete() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $this->filesystem->deleteFile( $this->fullpath );   
        } catch ( Exception $e ) {
            $this->flash['error'] = $i18n->__( 'file_not_deleted', basename( $this->decodedPath ) );
        } finally {
            $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( dirname( $this->decodedPath ) ) );
        }
    }

    public function doRename() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $filename = $this->request->getParameters()->getString( 'new_filename', null, 'unsafe_raw', FILTER_FLAG_STRIP_LOW );
            $this->filesystem->renameFile( $this->fullpath, dirname( $this->fullpath ) . DS . $filename );
        } catch ( exceptions\FileExistsException $e ) {
            $this->flash['error'] = $i18n->__( 'file_already_exists', $filename );
        } catch ( Exception $e ) {
            $this->flash['error'] = $i18n->__( 'file_not_renamed', $filename );
        } finally {
            $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( dirname( $this->decodedPath ) ) );
        }
    }

    public function doChangePermissions() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $this->filesystem->changePermissions( $this->fullpath, $this->request->getParameters()->getString( 'permissions', null, 'full_special_chars' ) );
        } catch ( Exception $e ) {
            $this->flash['error'] = $i18n->__( 'permissions_not_changed' );
        } finally {
            $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( dirname( $this->decodedPath ) ) );
        }
    }

    public function doUploadFile() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $this->filesystem->uploadFile( 'uploaded_file', $this->fullpath );            
        } catch ( \FileTooLargeException $e ) {
            $this->flash['error'] = $i18n->__('file_too_large' );
        } catch ( \InvalidFileFormatException $e ) {
            $this->flash['error'] = $i18n->__( 'invalid_file_format' );
        } catch ( exceptions\FileExistsException $e ) {
            $this->flash['error'] = $i18n->__( 'file_already_exists', '' );
        } catch ( Exception $e ) {
            $this->flash['error'] = $i18n->__( 'file_not_uploaded' );
        } finally {
            $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( $this->decodedPath ) );
        }
    }

    public function doViewFile() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $this->breadcrumbs = $this->directoryToBreadcrumb( $this->decodedPath );
            $this->file = $this->filesystem->readFile( $this->fullpath );
        } catch ( Exception $e ) {
            $this->flash['error'] = $i18n->__( 'cannot_open_file', basename( $this->decodedPath ) );
            $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( dirname( $this->decodedPath ) ) );
        }
        if ( in_array( $this->file['mime_type'], ['image/jpeg', 'image/png', 'image/gif', 'image/bmp', 'image/x-icon'] ) ) {
            $this->render( 'show_image.html' );
        }
    }

    public function doSaveFile() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $this->filesystem->writeFile( $this->fullpath, $this->request->getParameter( 'file_content' ) );
            $this->flash['success'] = $i18n->__( 'file_successfully_saved' );
        } catch ( exceptions\FileNotWritableException $e ) {
            $this->flash['error'] = $i18n->__( 'file_not_writable' );
        }
        catch ( RuntimeException $e ) {
            $this->flash['error'] = $i18n->__( 'file_not_saved' );
        } finally {
            if ( $this->request->getParameters()->getBoolean( 'continue', false ) ) {
                $this->redirect( BASE_URL . 'file_manager/view_file/' . $this->encodedPath );
            } else {
                $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( dirname( $this->decodedPath ) ) );
            }
        }
    }
}
